#include "pitches.h"
 
// notes in the melody:
int songState = 0;
int melody[] = {
 NOTE_F4, NOTE_E4, NOTE_D4, NOTE_CS4,
 NOTE_C4, NOTE_B3, NOTE_AS3, NOTE_A3,
 NOTE_G3, NOTE_A3, NOTE_AS3, NOTE_A3, 
 NOTE_G3, NOTE_C4, 0, 
 
 NOTE_C4, NOTE_A3, NOTE_A3, NOTE_A3,
 NOTE_GS3, NOTE_A3, NOTE_F4, NOTE_C4, 
 NOTE_C4, NOTE_A3, NOTE_AS3, NOTE_AS3, 
 NOTE_AS3, NOTE_C4, NOTE_D4, 0, 
 
 NOTE_AS3, NOTE_G3, NOTE_G3, NOTE_G3,
 NOTE_FS3, NOTE_G3, NOTE_E4, NOTE_D4, 
 NOTE_D4, NOTE_AS3, NOTE_A3, NOTE_A3, 
 NOTE_A3, NOTE_AS3, NOTE_C4, 0,
 
 NOTE_C4, NOTE_A3, NOTE_A3, NOTE_A3,
 NOTE_GS3, NOTE_A3, NOTE_A4, NOTE_F4, 
 NOTE_F4, NOTE_C4, NOTE_B3, NOTE_G4, 
 NOTE_G4, NOTE_G4, NOTE_G4, 0,
 
 NOTE_G4, NOTE_E4, NOTE_G4, NOTE_G4,
 NOTE_FS4, NOTE_G4, NOTE_D4, NOTE_G4, 
 NOTE_G4, NOTE_FS4, NOTE_G4, NOTE_C4, 
 NOTE_B3, NOTE_C4, NOTE_B3, NOTE_C4, 0
};
int tempo[] = {
 8, 16, 8, 16,
 8, 16, 8, 16,
 16, 16, 16, 8,
 16, 8, 3,
 
 12, 16, 16, 16,
 8, 16, 8, 16,
 8, 16, 8, 16,
 8, 16, 4, 12,
 
 12, 16, 16, 16,
 8, 16, 8, 16,
 8, 16, 8, 16,
 8, 16, 4, 12,
 
 12, 16, 16, 16,
 8, 16, 8, 16,
 8, 16, 8, 16,
 8, 16, 4, 16,
 
 12, 17, 17, 17,
 8, 12, 17, 17, 
 17, 8, 16, 8,
 16, 8, 16, 8, 1 
};

// music
unsigned long previousMillis2 = 0; // time last changed
const long interval2 = 100; // interval between notes

// pins
const int startGamePin = 12;
const int stopGamePin = 11;
const int buzzerPin = 10;
const int circuitPin = 9;
const int redPin = 6;
const int greenPin = 5;
const int timerPinsCount = 10;
const int timerPins[] = {4, 3, 2, 7, A0, A1, A2, A3, A4, A5};

// leds
int redValue = 0;
int greenValue = 0;
int blueValue = 0;
int colorId = 0;
int reds[] = {0, 0, 255, 255};
int greens[] = {0, 255, 140, 0};

// timer
unsigned long previousMillis1 = 0; // time last changed
const long interval1 = 5000; // interval between notes
int timerId = 9;

// game logic
int gameStarted = 0;
int hitWire = 0;
int highScore = -1;

void buzz(int targetPin, long frequency, long length) {
  long delayValue = 1000000/frequency/2; // calculate the delay value between transitions
  long numCycles = frequency * length/ 1000; // calculate the number of cycles for proper timing
  
  for (long i=0; i < numCycles; i++){
    digitalWrite(targetPin,HIGH);
    delayMicroseconds(delayValue);
    digitalWrite(targetPin,LOW); 
    delayMicroseconds(delayValue);
  }
}
void sing(unsigned long currentMillis) {
 // play the song in a non blocking way
 if (currentMillis - previousMillis2 >= interval2) {
   previousMillis2 = currentMillis;
   int noteDuration = 1000 / tempo[songState];
   buzz(buzzerPin, melody[songState], noteDuration);
   int pauseBetweenNotes = noteDuration;
   delay(pauseBetweenNotes);
   // stop the tone playing:
   buzz(buzzerPin, 0, noteDuration);
   
   ++songState;
   // start song again if finished
   if(songState > 79) {
    songState = 14; // skip intro 
   } 
 }
}

void updateTimer(unsigned long currentMillis) {
  // update timer in a non blocking way
  if (currentMillis - previousMillis1 >= interval1) {
    previousMillis1 = currentMillis;
    analogWrite(timerPins[timerId], 0);
    timerId--;

    if (timerId < 0) {
      gameStarted = 0;
      colorId = -1;
      hitWire = 1;
    }    
  }
}

void setup() {
 pinMode(startGamePin, INPUT_PULLUP);
 pinMode(stopGamePin, INPUT_PULLUP);
 pinMode(buzzerPin, OUTPUT); // setup buzzer 1
 pinMode(circuitPin, INPUT_PULLUP);
 pinMode(redPin, OUTPUT);
 pinMode(greenPin, OUTPUT);
 digitalWrite(redPin, LOW);
 digitalWrite(greenPin, LOW);
 gameStarted = 0;

 for (int i = 0; i < timerPinsCount; i++) {
  pinMode(timerPins[i], OUTPUT);
  analogWrite(timerPins[i], 0);
 }
}
 
void loop() { 

  if(digitalRead(startGamePin) == LOW) {
    // set start game data
    if (gameStarted == 1)
      return;
    gameStarted = 1;
    songState = 0;
    colorId = 0;
    hitWire = 1;
    for (int i = 0; i < timerPinsCount; i++) {
      pinMode(timerPins[i], OUTPUT);
      analogWrite(timerPins[i], 255);
    }
    timerId = timerPinsCount - 1;
    previousMillis1 = millis();
  } else if (digitalRead(stopGamePin) == LOW) {
    // set top game data
    gameStarted = 0;
    colorId = -1;
    hitWire = 1;
    if (timerId > highScore) {
      highScore = timerId;
    }
  }

  // led color management, updates when wire is touched
  if (hitWire == 1) {
    hitWire = 0;
    colorId++;

    if (colorId > 3) {
      gameStarted = 0;
      colorId = 0;
    }
    
    analogWrite(redPin, reds[colorId]);
    analogWrite(greenPin, greens[colorId]);
  }

  if (gameStarted == 0) {

    // print blinking high score
    if (highScore > 0) {
      delay(1000);
      for (int i = 0; i <= highScore; i++) {
        analogWrite(timerPins[i], 255);
      }
      for (int i = highScore + 1; i < timerPinsCount; i++) {
        analogWrite(timerPins[i], 0);
      }
      delay(1000);
      for (int i = 0; i < timerPinsCount; i++) {
        analogWrite(timerPins[i], 0);
      }
    }
    
    return;
  }

  // buzzer management
  if(digitalRead(circuitPin) == LOW) {
    delay(25);
    while(digitalRead(circuitPin) == LOW) {
      buzz(buzzerPin, NOTE_B0, 1000/24);
      hitWire = 1;
    }
  }

  unsigned long currentMillis = millis();
  updateTimer(currentMillis);
  sing(currentMillis);
}
